package ch.epfl.labos.iu.orm.queryll2;

import org.jinq.orm.annotations.EntitySupplier;

import ch.epfl.labos.iu.orm.query2.EntityManagerBackdoor;
import ch.epfl.labos.iu.orm.query2.SQLQuery;
import ch.epfl.labos.iu.orm.queryll2.path.Annotations;
import ch.epfl.labos.iu.orm.queryll2.path.StaticMethodAnalysisStorage;
import ch.epfl.labos.iu.orm.queryll2.symbolic.MethodCallValue;
import ch.epfl.labos.iu.orm.queryll2.symbolic.MethodSignature;
import ch.epfl.labos.iu.orm.queryll2.symbolic.TypedValue;
import ch.epfl.labos.iu.orm.queryll2.symbolic.TypedValueVisitor;
import ch.epfl.labos.iu.orm.queryll2.symbolic.TypedValueVisitorException;

public class SymbExToSubQueryGenerator<T> extends TypedValueVisitor<T, SQLQuery, TypedValueVisitorException>
{
   ORMInformation entityInfo;
   SymbExLambdaContext<T> lambdaContext;
   StaticMethodAnalysisStorage queryMethodHandler;

   public SymbExToSubQueryGenerator(ORMInformation entityInfo, SymbExArgHandler<T> argHandler, SymbExGetFieldHandler<T> getFieldHandler, SymbExJoinHandler<T> joinHandler, StaticMethodAnalysisStorage queryMethodHandler) 
   {
      this.entityInfo = entityInfo;
      this.lambdaContext = new SymbExLambdaContext<T>(argHandler, getFieldHandler, joinHandler);
      this.queryMethodHandler = queryMethodHandler;
   }
   public SQLQuery generateFor(TypedValue val) throws TypedValueVisitorException
   {
      return val.visit(this, null);
   }

   @Override public SQLQuery defaultValue(TypedValue val, T in) throws TypedValueVisitorException
   {
      throw new TypedValueVisitorException("Unhandled symbolic execution operation: " + val);
   }

   @Override public SQLQuery argValue(TypedValue.ArgValue val, T in) throws TypedValueVisitorException
   {
      if (lambdaContext.args != null)
         return lambdaContext.args.argSubQueryValue(val, in);
      return super.argValue(val, in);
   }

   @Override public SQLQuery getFieldValue(TypedValue.GetFieldValue val, T in) throws TypedValueVisitorException
   {
      if (lambdaContext.fields != null)
         return lambdaContext.fields.getFieldSubQueryValue(val, in);
      return super.getFieldValue(val, in);
   }

   EntityManagerBackdoor getAndCheckEntityManager(TypedValue val)
   {
      if (lambdaContext.fields != null)
         return lambdaContext.fields.getAndCheckEntityManager(val);
      return null;
   }
   
   @Override public SQLQuery virtualMethodCallValue(MethodCallValue.VirtualMethodCallValue val, T in) throws TypedValueVisitorException
   {
      MethodSignature sig = val.getSignature();
      if (entityInfo.allEntityMethods.containsKey(sig))
      {
         String entityName = entityInfo.allEntityMethods.get(sig);
         EntityManagerBackdoor em = getAndCheckEntityManager(val.base);
         if (em == null)
            throw new TypedValueVisitorException("Accessing an unknown entity manager");
         return new SQLQuery.SelectFromWhere<T>(
               em.getReaderForEntity(entityName), 
               em.getEntityColumnNames(entityName),
               em.getTableForEntity(entityName));
      }
      else
      {
         EntitySupplier entitySupplier = Annotations.methodFindAnnotation(sig, EntitySupplier.class);
         if (entitySupplier != null)
         {
            String entityName = entitySupplier.entityClass();
            // TODO: Fix this with a proper mapping between class name and entity
            entityName = entityName.substring(entityName.lastIndexOf(".") + 1);
            EntityManagerBackdoor em = getAndCheckEntityManager(val.base);
            if (em == null)
               throw new TypedValueVisitorException("Accessing an unknown entity manager");
            return new SQLQuery.SelectFromWhere<T>(
                  em.getReaderForEntity(entityName), 
                  em.getEntityColumnNames(entityName),
                  em.getTableForEntity(entityName));
         }
         else
            return super.virtualMethodCallValue(val, in);
      }
   }
}
